/*
 * *********************************************************************************************************************************************
 * "THE BEER-WARE LICENSE" (Revision 42):
 * Mario Hellmich <mario.hellmich@web.de> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * Mario Hellmich
 * *********************************************************************************************************************************************
 * def.h
 *
 * Version 1.0
 *
 * Created: 01.07.2018
 * Author: Mario Hellmich <mario.hellmich@web.de>
 * Project: HF Receiver Mark 2 Firmware
 *
 * This file contains definitions of the individual bits in instr_state and instr_error, the front panel key codes, and some constants
 * that determine the fundamental operation of the receiver (maximal and minimal tunable frequency, maximal and minimal COR level,
 * and maximal and minimal COR level.
 */ 


// Define asm NOP and CPU clock
#define F_CPU 16000000UL																	// CPU clock is 16 MHz
#define NOP() asm volatile("nop")


// Instrument state bits in instr_state
#define FM					0
#define AM					1
#define USB					2
#define LSB					3
#define MUTE_RCVR			4
#define SPEAKER_ON			5
#define AGC_FAST			6
#define AGC_SLOW			7
#define AGC_HANG			8
#define MGC					9
#define BW1					10
#define BW2					11
#define BW3					12
#define BW4					13
#define TUNE_100KHZ			14
#define TUNE_10KHZ			15
#define TUNE_1KHZ			16
#define TUNE_100HZ			17
#define TUNE_10HZ			18
#define TUNE_1HZ			19
#define BFOTUNE_100HZ		20
#define BFOTUNE_10HZ		21
#define BFOTUNE_1HZ			22
#define SET_CORLVL			23
#define COR_ACTIVE			24
#define BFO_TUNE			25
#define ATT_IN				26
#define PREAMP_IN			27
#define PRESEL_IN			28
#define SHIFT				29
#define KHZ_LED				30
#define MHZ_LED				31


// Front panel key codes																	   Component designator on A3 board:
#define KEY_AM				1																// SW1
#define KEY_USB				2																// SW2
#define KEY_LSB				3																// SW3
#define KEY_FM				4																// SW4
#define KEY_FAST			5																// SW5
#define KEY_SLOW			6																// SW6
#define KEY_HANG			7																// SW7
#define KEY_MGC				8																// SW8
#define KEY_BW1				9																// SW9
#define KEY_BW2				10																// SW10
#define KEY_BW3				11																// SW11
#define KEY_BW4				12																// SW12
#define KEY_1HZ				13																// SW13
#define KEY_10HZ			14																// SW14
#define KEY_100HZ			15																// SW15
#define KEY_1KHZ			16																// SW16
#define KEY_10KHZ			17																// SW17
#define KEY_100KHZ			18																// SW18
#define KEY_SETCOR			19																// SW19
#define KEY_BFOTUNE			20																// SW20
#define KEY_ATT				21																// SW21
#define KEY_PRESEL			22																// SW22
#define KEY_PREAMP			23																// SW23
#define KEY_SPKR			24																// SW24
#define KEY_SHIFT			25																// SW25


// Outputs port extender shift register on A2
// Byte 0
#define PEXT_BW1			0
#define PEXT_BW2			1
#define PEXT_BW3			2
#define PEXT_BW4			3
#define PEXT_AM				4
#define PEXT_SSB			5
#define PEXT_FM				6
#define PEXT_USB			7
// Byte 1
#define PEXT_AGC_AUTO		0
#define PEXT_AGC_FAST		1
#define PEXT_AGC_SLOW		2
#define PEXT_AGC_HANG		3
#define PEXT_ATT_OUT		4
#define PEXT_PREAMP_ON		5
#define PEXT_SPKR_OFF		6
// Byte 2
#define PEXT_PS_BD1			0
#define PEXT_PS_BD2			1
#define PEXT_PS_BD3			2
#define PEXT_PS_BD4			3
#define PEXT_PS_BD5			4
#define PEXT_PS_BD6			5
#define PEXT_PS_OFF			6


// Error code bits in instr_error
#define ERR_RSSI_RANGE		0																// RSSI voltage out of permissible range
#define ERR_PLL_LOL			1																// Loss of lock error in PLL synthesizer
#define ERR_STUCK_KEY		2																// Stuck key at startup
#define ERR_POS_5V_ANALOG	3																// +5VA not present or out of permissible range
#define ERR_NEG_5V_ANALOG	4																// -5VA not present or out of permissible range


// Constants
#define MIN_COR_LEVEL		120																// Minimum COR level is -120 dBm, below that COR is always enabled
#define MAX_COR_LEVEL		0																// Maximum COR level is 0 dBm
#define MAX_FREQ			30000000														// Max frequency (in Hz) the receiver can be tuned to, i.e. 30.000,000 MHz, tuning and display resolution 1 Hz
#define MIN_FREQ			1000															// Min Frequency (in Hz) the receiver can be tuned to, i.e. 00.001,000 MHz
#define MAX_BFO_OFFS		3500															// Max BFO offset (in Hz) that can be selected
#define MIN_BFO_OFFS		-3500															// Min BFO offset (in Hz) that can be selected


// Preselector band limits																				band end frequency
#define PRESEL_BL_1			1500000															// Band 1:  1.5 MHz
#define PRESEL_BL_2			3000000															// Band 2:  3   MHz
#define PRESEL_BL_3			6000000															// Band 3:  6   MHz
#define PRESEL_BL_4			12000000														// Band 4: 12   MHz
#define PRESEL_BL_5			24000000														// Band 5: 24   MHz
																							// Band 6: 32   MHz