/*
 * *********************************************************************************************************************************************
 * "THE BEER-WARE LICENSE" (Revision 42):
 * Mario Hellmich <mario.hellmich@web.de> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * Mario Hellmich
 * *********************************************************************************************************************************************
 * twi.c
 *
 * Version 1.0
 *
 * Created: 01.07.2018
 * Author: Mario Hellmich <mario.hellmich@web.de>
 * Project: HF Receiver Mark 2 Firmware
 *
 * Routines for controlling the TWI bus
 */ 


#include "def.h"

#include <avr/io.h>
#include <avr/interrupt.h>

#include "twi.h"




/*
************************************************************************************************************************************************
Basic TWI Bus Routines:
These routines are used to read and write on the TWI bus.
*/

void TWI_init(void)
{
	TWSR  = 0x00;																			// Set SCL to 400 kHz
	TWBR  = 0x0C;
	TWCR  = (1<<TWEN);																		// TWI enable
}

void TWI_start(void)																		// Send TWI start condition
{
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
	while( !(TWCR & (1 << TWINT)) );
}

void TWI_stop(void)																			// Send TWI stop condition
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
}

void TWI_write(uint8_t data)																// Write one byte to the TWI bus
{
	TWDR = data;																			// Load data into TWDR register
	TWCR = (1<<TWINT) | (1<<TWEN);															// Set TWINT bit (i.e. clear flag) in TWCR to start transmission
	while( !(TWCR & (1<<TWINT)) );															// Wait for TWINT flag, this indicates that data has been sent and ACK or NACK has been received
}

uint8_t TWI_read_ACK(void)																	// Read from TWI bus, return the TWDR register and send ACK
{
	TWCR = (1<<TWINT) | (1<<TWEN)| (1<<TWEA);
	while ( !(TWCR & (1<<TWINT)) );
	return TWDR;
}

uint8_t TWI_read_NACK(void)																	// Read and return the TWDR register without sending ACK
{
	TWCR = (1<<TWINT) | (1<<TWEN);
	while ( !(TWCR & (1<<TWINT)) );
	return TWDR;
}

uint8_t TWI_get_status(void)																// Read and return the upper five bits of the TWI status register
{
	uint8_t status;
	status = TWSR & 0xF8;
	return status;
}